// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.IParameterValues;
import org.eclipse.ui.IEditorInput;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.AgentEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.GroupEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.NameEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.NameRelationshipEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.OccurrenceEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.ReferenceEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.UserEditorInput;

/**
 * <p>BulkEditorInputTypeValues class.</p>
 *
 * @author p.ciardelli
 * @created 20.08.2009
 * @version 1.0
 */
public class BulkEditorInputTypeValues implements IParameterValues {

	public enum BulkEditorInputType{
		REFERENCE("Reference", ReferenceEditorInput.ID),
		NAME("Name", NameEditorInput.ID),
		AGENT("Authors and Author Teams", AgentEditorInput.ID),
		OCCURRENCE("Specimens or Observations", OccurrenceEditorInput.ID), 
		NAME_RELATIONSHIP("Name Relationship", NameRelationshipEditorInput.ID),
		USER("User", UserEditorInput.ID), 
		GROUP("Group", GroupEditorInput.ID);
		
		
		public String id;
		public String label;
		
		BulkEditorInputType(String label, String id){
			this.id = id;
			this.label = label;
		}
		
		public static BulkEditorInputType getById(String id){
			
			for(BulkEditorInputType type : values()){
				if(id.equals(type.id)){
					return type;
				}
			}
			
			return null;
		}
		
		public static BulkEditorInputType getByType(Class clazz){
			if (Reference.class.isAssignableFrom(clazz)) {
				return REFERENCE;
			}
			else if (TaxonNameBase.class.isAssignableFrom(clazz)) {
				return NAME;
			}
			else if (AgentBase.class.isAssignableFrom(clazz)) {
				return AGENT;
			}
			else if (SpecimenOrObservationBase.class.isAssignableFrom(clazz)){
				return OCCURRENCE;
			}
			else if (NameRelationship.class.isAssignableFrom(clazz)){
				return NAME_RELATIONSHIP;
			}
			else if (Group.class.isAssignableFrom(clazz)){
				return GROUP;
			}
			else if (User.class.isAssignableFrom(clazz)){
				return USER;
			}
			return null;
		}
		
		public static BulkEditorInputType getByInput(IEditorInput input) {
			if (input instanceof ReferenceEditorInput) {
				return REFERENCE;
			}
			else if (input instanceof NameEditorInput) {
				return NAME;
			}
			else if (input instanceof AgentEditorInput) {
				return AGENT;
			}
			else if (input instanceof OccurrenceEditorInput) {
				return OCCURRENCE;
			}
			else if (input instanceof NameRelationshipEditorInput) {
				return NAME_RELATIONSHIP;
			}
			else if(input instanceof UserEditorInput){
				return USER;
			}
			else if(input instanceof GroupEditorInput){
				return GROUP;
			}
			return null;
		}
		
		public static AbstractBulkEditorInput getInput(BulkEditorInputType inputType){
			switch (inputType) {
			case REFERENCE:
				return new ReferenceEditorInput();
			case NAME:
				return new NameEditorInput();
			case AGENT:
				return new AgentEditorInput();
			case OCCURRENCE:
				return new OccurrenceEditorInput();
			case NAME_RELATIONSHIP:
				return new NameRelationshipEditorInput();
			case USER:
				return new UserEditorInput();
			case GROUP:
				return new GroupEditorInput();
			default:
				throw new IllegalStateException("No input class for the given input type defined.");
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IParameterValues#getParameterValues()
	 */
	/**
	 * <p>getParameterValues</p>
	 *
	 * @return a {@link java.util.Map} object.
	 */
	public Map getParameterValues() {
		final Map values = new HashMap();
		
		// TODO user role determines which editor inputs are returned
		for (BulkEditorInputType inputType : BulkEditorInputType.values()){
			values.put(inputType.label, inputType.id);
		}		
		
		return values;
	}
}
