// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;

import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created Mar 11, 2011
 * @version 1.0
 */
public class DeleteHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_EXPERIMENTAL_FEATURES)){
			ISelection selection = HandlerUtil.getCurrentSelection(event);
			
			IEditorPart editor = HandlerUtil.getActiveEditor(event);
			
			IEditorInput input = editor.getEditorInput();
			
			if((input instanceof IEntityPersistenceService) && (selection instanceof IStructuredSelection)){
				
				
				IDocumentProvider provider = ((BulkEditor) editor).getDocumentProvider();
				LineAnnotationModel model = 
						(LineAnnotationModel) provider.getAnnotationModel(input);
				
				
				IStructuredSelection structuredSelection = (IStructuredSelection) selection;
				
				IEntityPersistenceService persistenceService = (IEntityPersistenceService) input;
				
				for(Object object : structuredSelection.toList()){
					
					LineAnnotation annotation = (LineAnnotation) model.getAnnotation(object);
									
	//				persistenceService.delete(object);
					((BulkEditor) editor).removeAnnotatedLine(annotation);
					
				}				
			}
		}else{
			BulkEditorUtil.warningDialog("Feature not enabled", getClass(), "Deletion is currently an experimental feature." +
					"Enable it via Preferences at your own risk.");
		}
		
		
		return null;
	}

}
