// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldObservation;
import eu.etaxonomy.cdm.model.occurrence.Fossil;
import eu.etaxonomy.cdm.model.occurrence.LivingBeing;
import eu.etaxonomy.cdm.model.occurrence.Observation;
import eu.etaxonomy.cdm.model.occurrence.Specimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;

/**
 * <p>OccurrenceCreator class.</p>
 *
 * @author n.hoffmann
 * @created Jun 16, 2010
 * @version 1.0
 */
public class OccurrenceCreator implements IEntityCreator<SpecimenOrObservationBase> {
	private static final Logger logger = Logger
			.getLogger(OccurrenceCreator.class);

	/** {@inheritDoc} */
	public SpecimenOrObservationBase createEntity(String text) {
		return createEntity(Specimen.class, text);
	}

	/** {@inheritDoc} */
	public SpecimenOrObservationBase createEntity(Object key, String text) {
		SpecimenOrObservationBase specimenOrObservation = null;
		
		if (FieldObservation.class.equals(((Class) key))) {
			specimenOrObservation = FieldObservation.NewInstance();
		}
		else if (DerivedUnit.class.equals(((Class) key))) {
			specimenOrObservation = DerivedUnit.NewInstance();
		}
		else if (LivingBeing.class.equals(((Class) key))) {
			specimenOrObservation = LivingBeing.NewInstance();
		}
		else if (Observation.class.equals(((Class) key))) {
			specimenOrObservation = Observation.NewInstance();
		}
		else if (Specimen.class.equals(((Class) key))) {
			specimenOrObservation = Specimen.NewInstance();
		}
		else if (DnaSample.class.equals(((Class) key))) {
			specimenOrObservation = DnaSample.NewInstance();
		}
		else if (Fossil.class.equals(((Class) key))) {
			specimenOrObservation = Fossil.NewInstance();
		}
		
		if (specimenOrObservation != null) {
			specimenOrObservation.setTitleCache(text);
		}
		return specimenOrObservation;
	}

	/**
	 * <p>getKeyLabelPairs</p>
	 *
	 * @return a {@link java.util.Map} object.
	 */
	public Map<Object, String> getKeyLabelPairs() {
		Map<Object, String> result = new HashMap<Object, String>();
		result.put(FieldObservation.class, "Field Observation");
		result.put(DerivedUnit.class, "Derived Unit");
		result.put(LivingBeing.class, "Living Being");
		result.put(Observation.class, "Observation");
		result.put(Specimen.class, "Specimen");
		result.put(DnaSample.class, "Dna Sample");
		result.put(Fossil.class, "Fossil");
		return result;
	}
}
